﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Net;
using PLCCommLib;

namespace Sample
{
    /// <summary>PLC通信のサンプル</summary>
    public partial class SampleForm : Form
    {

        #region フィールド

        /// <summary>通信処理クラスのインスタンス</summary>
        private PLCComm _PLC = null;

        #endregion

        #region コンストラクタ

        /// <summary>既定のコンストラクタ</summary>
        public SampleForm()
        {
            InitializeComponent();

            // コントロールの有効・無効設定
            btnOpen.Enabled = true;
            btnClose.Enabled = false;
            grpParameter.Enabled = true;
            grpBlock.Enabled = false;
            grpBit.Enabled = false;

            // 設定の初期値
            cmbVender.SelectedIndex = 0;    // 三菱
            cmbProtocol.SelectedIndex = 0;  // MCプロトコル(3E)
            rdoTCP.Checked = true;
            rdoBIN.Checked = true;
        }

        #endregion

        #region プロパティ

        #endregion

        #region メソッド(Dummy)

        /// <summary>このままでは実行されないダミーのサンプルコード</summary>
        /// <remarks>ランダム読出</remarks>
        private void DummyMethod_Random()
        {
            List<string> dev = new List<string>();
            int size;
            short[] dat;
            int ret;

            // 通信クラスのインスタンス生成前は不可
            if (_PLC == null) return;

            // 通信回線オープンしないと通信は不可
            if (!_PLC.IsOpened()) return;


            // 読出サイズ
            // ※ワード単位の点数
            size = 4;
            dat = new short[size];

            // デバイス
            // ※PLCに合わせたワードデバイスを指定
            switch (_PLC.VenderName)
            {
                case PLCComm.VenderNames.MITSUBISHI:    // 三菱
                    dev.Add("X0");      // ビットのデバイス番号は16の倍数を推奨
                    dev.Add("M0");      // ビットのデバイス番号は16の倍数を推奨
                    dev.Add("D1000");
                    dev.Add("R5000");
                    break;
                case PLCComm.VenderNames.OMRON:         // OMRON
                    dev.Add("0");       // ワード形式で指定(.0～.15を付けない)
                    dev.Add("W0");      // ワード形式で指定(.0～.15を付けない)
                    dev.Add("D1000");
                    dev.Add("E5000");
                    break;
                case PLCComm.VenderNames.KEYENCE:       // KEYENCE
                    dev.Add("R000");    // ビットのデバイス番号下2桁(00～15のビット指定)は00を推奨
                    dev.Add("MR000");   // ビットのデバイス番号の下2桁(00～15のビット指定)は00を推奨
                    dev.Add("DM1000");
                    dev.Add("FM5000");
                    break;
                default:
                    return;
            }

            // ランダム読出
            ret = _PLC.ReadDeviceRandom(dev.ToArray(), size, ref dat);
            if (ret == PLCComm.RET_COMPLETED)
            {
                // 読出成功 → 戻り値が0
            }
            else
            {
                // 読出失敗 → 戻り値が0以外
                // 0x??000000の場合は通信ライブラリが返すエラーコード
                // 0x0000????の場合は通信プロトコル上でPLC側が返すエラーコード
                // ⇒エラーコードの詳細はプロトコルかEthernetユニットかCPUのいずれかのマニュアルに記載されているハズ
                Console.WriteLine("ReadDeviceRandom Fail. 0x" + ret.ToString("X8"));
            }

        }

        /// <summary>このままでは実行されないダミーのサンプルコード</summary>
        /// <remarks>ブロック読出→データ変換</remarks>
        private void DummyMethod_Block()
        {
            string dev;
            int size;
            short[] dat;
            int ret;

            // 通信クラスのインスタンス生成前は不可
            if (_PLC == null) return;

            // 通信回線オープンしないと通信は不可
            if (!_PLC.IsOpened()) return;


            // サイズ
            size = 2000;
            dat = new short[size];

            // デバイス
            // ※PLCに合わせたワードデバイスを指定
            switch (_PLC.VenderName)
            {
                case PLCComm.VenderNames.MITSUBISHI:    // 三菱
                    //dev = "X0";     // ビットのデバイス番号は16の倍数を推奨
                    dev = "D0";
                    break;
                case PLCComm.VenderNames.OMRON:         // OMRON
                    //dev = "0";      // ワード形式で指定(.0～.15を付けない)
                    dev = "D0";
                    break;
                case PLCComm.VenderNames.KEYENCE:       // KEYENCE
                    //dev = "R000";   // ビットのデバイス番号下2桁(00～15のビット指定)は00を推奨
                    dev = "DM0";
                    break;
                default:
                    return;
            }

            // ブロック読出
            ret = _PLC.ReadDeviceBlock(dev, size, ref dat);
            if (ret == PLCComm.RET_COMPLETED)
            {
                // 読出成功 → 戻り値が0
                int dword;
                float flt1;
                string ascii;

                // D100/D101のダブルワードに変換
                dword = DataConv.Word2DoubleWord(dat[100], dat[101]);

                // D500/D501の単精度浮動小数に変換
                flt1 = DataConv.Word2Single(dat[500], dat[501]);

                // D1000～のAscii文字列に変換
                ascii = DataConv.Word2Ascii(dat, false, 1000, -1);  // 下位Byte→上位Byteで格納される場合
                //ascii = DataConv.Word2Ascii(dat, true, 1000, -1);   // 上位Byte→下位Byteで格納される場合
                // ※PLC内で通常の文字列処理をする場合
                // 　　三菱:    下位→上位
                // 　　OMRON:   上位→下位
                // 　　KEYENCE: 上位→下位
                // 　で格納される。ただし上位通信との都合などにより逆にして使う場合もあり得る。

            }
            else
            {
                // 読出失敗 → 戻り値が0以外
                // 0x??000000の場合は通信ライブラリが返すエラーコード
                // 0x0000????の場合は通信プロトコル上でPLC側が返すエラーコード
                // ⇒エラーコードの詳細はプロトコルかEthernetユニットかCPUのいずれかのマニュアルに記載されているハズ
                Console.WriteLine("ReadDeviceBlock Fail. 0x" + ret.ToString("X8"));
            }

        }

        /// <summary>このままでは実行されないダミーのサンプルコード</summary>
        /// <remarks>ビット処理</remarks>
        private void DummyMethod_Bit()
        {
            string dev1, dev2;
            int ret;

            // 通信クラスのインスタンス生成前は不可
            if (_PLC == null) return;

            // 通信回線オープンしないと通信は不可
            if (!_PLC.IsOpened()) return;


            // デバイス
            // ※PLCに合わせたワードデバイスを指定
            // ※ワードデバイスのビット指定も可(タイマ・カウンタ現在値やインデックスは除く)
            // 　ただしMCプロトコルや上位リンクではプロトコル的にサポートされていないため
            // 　PLC側の処理内容によっては予期せぬ不具合を招く可能性がある
            switch (_PLC.VenderName)
            {
                case PLCComm.VenderNames.MITSUBISHI:    // 三菱
                    dev1 = "Y0";
                    dev2 = "D500.A";    // 三菱はビット番号を16進数で指定
                    break;
                case PLCComm.VenderNames.OMRON:         // OMRON
                    dev1 = "0.00";
                    dev2 = "D500.10";   // OMRONはビット番号を10進数で指定
                    break;
                case PLCComm.VenderNames.KEYENCE:       // KEYENCE
                    dev1 = "R000";
                    dev2 = "DM500.10";  // KEYENCEはビット番号を10進数で指定
                    break;
                default:
                    return;
            }

            // ビットSET(1)
            ret = _PLC.SetBit(dev1);

            // ビットRESET(0)
            ret = _PLC.ResetBit(dev1);

            // ビット反転
            ret = _PLC.InverseBit(dev2);
            // ※通信ライブラリ側で読出→論理演算→SET or RESETの順に処理
            // 　PLC側の処理内容によっては正常に動作しない可能性がある

        }

        #endregion

        #region イベント処理

        /// <summary>通信回線接続ボタンクリック</summary>
        /// <remarks>通信設定チェック→通信クラスのインスタンス生成→オープン→コントロールの有効・無効設定</remarks>
        private void btnOpen_Click(object sender, EventArgs e)
        {
            PLCComm.VenderNames vnd = (PLCComm.VenderNames)cmbVender.SelectedIndex;
            PLCComm.ProtocolNames pro = (PLCComm.ProtocolNames)cmbProtocol.SelectedIndex;
            string ip;
            IPAddress ipadr = new IPAddress(0);
            int port;

            // ベンダとプロトコルの組合せ
            switch (vnd)
            {
                case PLCComm.VenderNames.MITSUBISHI:
                    if (pro != PLCComm.ProtocolNames.MC3E) return;
                    break;

                case PLCComm.VenderNames.OMRON:
                    if (pro != PLCComm.ProtocolNames.FINS) return;
                    break;

                case PLCComm.VenderNames.KEYENCE:
                    if (pro != PLCComm.ProtocolNames.HiLink && pro != PLCComm.ProtocolNames.MC3E) return;
                    break;

                default:
                    return;
            }

            // IPアドレス
            ip = txtIP0.Text + "." + txtIP1.Text + "." + txtIP2.Text + "." + txtIP3.Text;
            if (!IPAddress.TryParse(ip, out ipadr)) return;

            // 通信ポート
            if (!int.TryParse(txtPort.Text, out port)) return;



            // 通信処理のインスタンスを生成
            switch (vnd)
            {
                case PLCComm.VenderNames.MITSUBISHI:
                    _PLC = new PLC_MC();
                    break;

                case PLCComm.VenderNames.OMRON:
                    _PLC = new PLC_Fins();
                    break;

                case PLCComm.VenderNames.KEYENCE:
                    if (pro == PLCComm.ProtocolNames.HiLink)
                        _PLC = new PLC_HiLink();
                    else if (pro == PLCComm.ProtocolNames.MC3E)
                        _PLC = new PLC_MCKV();
                    else
                        return;
                    break;

                default:
                    return;
            }

            // 設定 ※必要に応じて変更
            //_PLC.Timeout = 3000;
            _PLC.IsUdp = rdoUDP.Checked;
            _PLC.IsAscii = rdoASC.Checked;
            _PLC.IPAddress = ip;
            _PLC.PortNumber = port;
            //// ↓MCプロトコル用↓
            //if (_PLC.ProtocolName == PLCComm.ProtocolNames.MC3E)
            //{
            //    _PLC.NetworkNumber = 0;
            //    _PLC.PCNumber = 0xFF;
            //    _PLC.ReqUnitIONumber = 0x3FF;
            //    _PLC.ReqUnitStnNumber = 0;
            //    _PLC.CPUCheckTime = 8;
            //}
            //// ↓FINS用↓
            //if (_PLC.ProtocolName == PLCComm.ProtocolNames.FINS)
            //{
            //    _PLC.DNA = 0;
            //    _PLC.DA1 = 0;
            //    _PLC.DA2 = 0;
            //}



            // 回線オープン
            int ret  = _PLC.Open();
            if (ret == PLCComm.RET_COMPLETED)
            {
                // 成功。
                btnOpen.Enabled = false;
                btnOpen.BackColor = Color.Lime;
                btnClose.Enabled = true;
                grpParameter.Enabled = false;
                grpBlock.Enabled = true;
                grpBit.Enabled = true;

                // デバイステキストボックス用のツールチップ
                toolTip1.RemoveAll();
                StringBuilder tip = new StringBuilder(1024);
                StringBuilder tipbit = new StringBuilder(1024);
                PLCComm.DeviceInformation[] info;
                info = _PLC.GetAvailableDeviceInfo();
                for (int i = 0; i < info.Length; i++)
                {
                    tip.AppendLine(info[i].Name + ": " + info[i].Description);
                    if (info[i].fBit || info[i].fWordBit)
                        tipbit.AppendLine(info[i].Name + ": " + info[i].Description);
                }
                toolTip1.SetToolTip(txtDevBlock, tip.ToString());
                toolTip1.SetToolTip(txtDevBit, tipbit.ToString());
            }
            else
            {
                // 失敗。
                Console.WriteLine("Open Fail... 0x" + ret.ToString("X8"));
            }

        }

        /// <summary>通信回線切断ボタンクリック</summary>
        /// <remarks>クローズ→コントロールの有効・無効設定</remarks>
        private void btnClose_Click(object sender, EventArgs e)
        {
            if (_PLC == null) return;

            _PLC.Close();
            _PLC = null;

            btnOpen.Enabled = true;
            btnOpen.BackColor = SystemColors.Control;
            btnClose.Enabled = false;
            grpParameter.Enabled = true;
            grpBlock.Enabled = false;
            grpBit.Enabled = false;
        }

        /// <summary>ブロック読出ボタンクリック</summary>
        /// <remarks>指定の先頭デバイス(ワード)から指定サイズ(点数)の読出→データ表示</remarks>
        private void btnReadBlock_Click(object sender, EventArgs e)
        {
            string dev;
            int size;
            short[] dat;
            int ret;

            // チェック
            if (_PLC == null) return;
            dev = _PLC.RemakeDevice(txtDevBlock.Text, 0);
            if (string.IsNullOrEmpty(dev)) return;
            //if (!_PLC.IsWordAvailable(dev)) return;             // ←コレでチェックするとビットデバイス指定時に制約が増える
            if (!int.TryParse(txtSize.Text, out size)) return;
            if (size < 1) return;

            // 読出
            dat = new short[size];
            ret = _PLC.ReadDeviceBlock(dev, size, ref dat);

            if (ret == PLCComm.RET_COMPLETED)
            {
                // 読出成功
                StringBuilder txt = new StringBuilder(2048);
                txt.AppendLine(dev + ", " + size.ToString());
                txt.AppendLine("Read Time = " + _PLC.GetLastProcessingTime().ToString() + "ms");
                for (int i = 0; i < size; i++)
                {
                    if (i >= 50)
                    {
                        txt.AppendLine("...");
                        break;
                    }
                    txt.AppendLine("[" + i.ToString("D2") + "] = " + dat[i].ToString());
                }
                txtReadData.Text = txt.ToString();

            }
            else
            {
                // 読出失敗
                txtReadData.Text = "Read Fail...\r\n0x" + ret.ToString("X8");
            }

        }

        /// <summary>ブロック書込ボタンクリック</summary>
        /// <remarks>指定の先頭デバイス(ワード)から指定サイズ(点数)の書込</remarks>
        private void btnWriteBlock_Click(object sender, EventArgs e)
        {
            string dev;
            int size;
            short[] dat;
            int ret;

            // チェック
            if (_PLC == null) return;
            dev = _PLC.RemakeDevice(txtDevBlock.Text, 0);
            if (string.IsNullOrEmpty(dev)) return;
            //if (!_PLC.IsWordAvailable(dev)) return;             // ←コレでチェックするとビットデバイス指定時に制約が増える
            if (!int.TryParse(txtSize.Text, out size)) return;
            if (size < 1) return;

            // 書込
            dat = new short[size];  // 配列確保で要素は全て0
            ret = _PLC.WriteDeviceBlock(dev, size, ref dat);

            if (ret != PLCComm.RET_COMPLETED)
                Console.WriteLine("Write Fail... 0x" + ret.ToString("X8"));

        }

        /// <summary>SETボタンクリック</summary>
        /// <remarks>指定のビットデバイス(ワードデバイスのビットも可)をSET</remarks>
        private void btnBitSet_Click(object sender, EventArgs e)
        {
            string dev;
            int ret;

            // チェック
            if (_PLC == null) return;
            dev = _PLC.RemakeDevice(txtDevBit.Text, 0);
            if (string.IsNullOrEmpty(dev)) return;
            if (!_PLC.IsBitDevice(dev)) return;

            // ビットセット
            ret = _PLC.SetBit(dev);
            if (ret != PLCComm.RET_COMPLETED)
                Console.WriteLine("SET Fail... 0x" + ret.ToString("X8"));

        }

        /// <summary>RSTボタンクリック</summary>
        /// <remarks>指定のビットデバイス(ワードデバイスのビットも可)をRESET</remarks>
        private void btnBitReset_Click(object sender, EventArgs e)
        {
            string dev;
            int ret;

            // チェック
            if (_PLC == null) return;
            dev = _PLC.RemakeDevice(txtDevBit.Text, 0);
            if (string.IsNullOrEmpty(dev)) return;
            if (!_PLC.IsBitDevice(dev)) return;

            // ビットリセット
            ret = _PLC.ResetBit(dev);
            if (ret != PLCComm.RET_COMPLETED)
                Console.WriteLine("RST Fail... 0x" + ret.ToString("X8"));

        }

        /// <summary>反転ボタンクリック</summary>
        /// <remarks>指定のビットデバイス(ワードデバイスのビットも可)を反転</remarks>
        private void btnBitInverse_Click(object sender, EventArgs e)
        {
            string dev;
            int ret;

            // チェック
            if (_PLC == null) return;
            dev = _PLC.RemakeDevice(txtDevBit.Text, 0);
            if (string.IsNullOrEmpty(dev)) return;
            if (!_PLC.IsBitDevice(dev)) return;

            // ビット反転
            ret = _PLC.InverseBit(dev);
            if (ret != PLCComm.RET_COMPLETED)
                Console.WriteLine("INV Fail... 0x" + ret.ToString("X8"));

        }



        /// <summary>フォームクローズ</summary>
        /// <remarks>通信回線切断</remarks>
        private void SampleForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_PLC != null) _PLC.Close();
        }



        /// <summary>デバイス入力テキストボックスのマウスホイール</summary>
        /// <remarks>ホイールに合わせてデバイスをINC・DEC</remarks>
        private void txtDevBlock_MouseWheel(object sender, MouseEventArgs e)
        {
            TextBox txt = (TextBox)sender;
            string dev = txt.Text;
            int div = e.Delta / SystemInformation.MouseWheelScrollDelta; // ← 通常±3の範囲になるハズ

            // デバイスと認識できないものは処理しない
            if (string.IsNullOrEmpty(dev)) return;
            if (txt.Name == "txtDevBlock")
            {
                if (!_PLC.IsDevice(dev)) return;
            }
            else
            {
                if (!_PLC.IsBitDevice(dev)) return;
            }

            // ホイール回転分INC/DEC
            dev = _PLC.RemakeDevice(dev, div);
            if (string.IsNullOrEmpty(dev)) return;

            // カーソル位置を合わせてTextを変更
            int selLast = txt.Text.Length - txt.SelectionStart;
            txt.Text = dev;
            if (txt.Text.Length > selLast) txt.SelectionStart = txt.Text.Length - selLast;

        }

        /// <summary>デバイス入力テキストボックスのキーダウン</summary>
        /// <remarks>Ctrl+Shift+Up/DownでデバイスをINC・DEC</remarks>
        private void txtDevBlock_KeyDown(object sender, KeyEventArgs e)
        {
            TextBox txt = (TextBox)sender;
            string dev = txt.Text;
            int div;

            // デバイスと認識できないものは処理しない
            if (string.IsNullOrEmpty(dev)) return;
            if (txt.Name == "txtDevBlock")
            {
                if (!_PLC.IsDevice(dev)) return;
            }
            else
            {
                if (!_PLC.IsBitDevice(dev)) return;
            }

            // Up or DownキーでINC or DEC
            if (e.Control && e.Shift)
            {
                if (e.KeyCode == Keys.Up)
                    div = 1;
                else if (e.KeyCode == Keys.Down)
                    div = -1;
                else
                    return;
            }
            else
            {
                return;
            }
            dev = _PLC.RemakeDevice(dev, div);
            if (string.IsNullOrEmpty(dev)) return;

            // カーソル位置を合わせてTextを変更
            int selLast = txt.Text.Length - txt.SelectionStart;
            txt.Text = dev;
            if (txt.Text.Length > selLast) txt.SelectionStart = txt.Text.Length - selLast;

            e.Handled = true;
        }

        #endregion

    }
}
